@@ -59,40 +59,44 @@ use Apache::TestRun ();
 use vars qw(%Usage);
 
 %Usage = (
-   top_dir         => 'top-level directory (default is $PWD)',
-   t_dir           => 'the t/ test directory (default is $top_dir/t)',
-   t_conf          => 'the conf/ test directory (default is $t_dir/conf)',
-   t_logs          => 'the logs/ test directory (default is $t_dir/logs)',
-   t_pid_file      => 'location of the pid file (default is $t_logs/httpd.pid)',
-   t_conf_file     => 'test httpd.conf file (default is $t_conf/httpd.conf)',
-   src_dir         => 'source directory to look for mod_foos.so',
-   serverroot      => 'ServerRoot (default is $t_dir)',
-   documentroot    => 'DocumentRoot (default is $ServerRoot/htdocs',
-   port            => 'Port [port_number|select] (default ' . DEFAULT_PORT . ')',
-   servername      => 'ServerName (default is localhost)',
-   user            => 'User to run test server as (default is $USER)',
-   group           => 'Group to run test server as (default is $GROUP)',
-   bindir          => 'Apache bin/ dir (default is apxs -q BINDIR)',
-   sbindir         => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
-   httpd           => 'server to use for testing (default is $bindir/httpd)',
-   target          => 'name of server binary (default is apxs -q TARGET)',
-   apxs            => 'location of apxs (default is from Apache2::BuildConfig)',
-   startup_timeout => 'seconds to wait for the server to start (default is 60)',
-   httpd_conf      => 'inherit config from this file (default is apxs derived)',
-   httpd_conf_extra=> 'inherit additional config from this file',
-   minclients      => 'minimum number of concurrent clients (default is 1)',
-   maxclients      => 'maximum number of concurrent clients (default is minclients+1)',
-   perlpod         => 'location of perl pod documents (for testing downloads)',
-   proxyssl_url    => 'url for testing ProxyPass / https (default is localhost)',
-   sslca           => 'location of SSL CA (default is $t_conf/ssl/ca)',
-   sslcaorg        => 'SSL CA organization to use for tests (default is asf)',
-   libmodperl      => 'path to mod_perl\'s .so (full or relative to LIBEXECDIR)',
-   defines         => 'values to add as -D defines (for example, "VAR1 VAR2")',
+   top_dir          => 'top-level directory (default is $PWD)',
+   t_dir            => 'the t/ test directory (default is $top_dir/t)',
+   t_conf           => 'the conf/ test directory (default is $t_dir/conf)',
+   t_logs           => 'the logs/ test directory (default is $t_dir/logs)',
+   t_state          => 'the state/ test directory (default is $t_dir/state)',
+   t_pid_file       => 'location of the pid file (default is $t_logs/httpd.pid)',
+   t_conf_file      => 'test httpd.conf file (default is $t_conf/httpd.conf)',
+   src_dir          => 'source directory to look for mod_foos.so',
+   serverroot       => 'ServerRoot (default is $t_dir)',
+   documentroot     => 'DocumentRoot (default is $ServerRoot/htdocs',
+   port             => 'Port [port_number|select] (default ' . DEFAULT_PORT . ')',
+   servername       => 'ServerName (default is localhost)',
+   user             => 'User to run test server as (default is $USER)',
+   group            => 'Group to run test server as (default is $GROUP)',
+   bindir           => 'Apache bin/ dir (default is apxs -q BINDIR)',
+   sbindir          => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
+   httpd            => 'server to use for testing (default is $bindir/httpd)',
+   target           => 'name of server binary (default is apxs -q TARGET)',
+   apxs             => 'location of apxs (default is from Apache2::BuildConfig)',
+   startup_timeout  => 'seconds to wait for the server to start (default is 60)',
+   httpd_conf       => 'inherit config from this file (default is apxs derived)',
+   httpd_conf_extra => 'inherit additional config from this file',
+   minclients       => 'minimum number of concurrent clients (default is 1)',
+   maxclients       => 'maximum number of concurrent clients (default is minclients+1)',
+   threadsperchild  => 'number of threads per child when using threaded MPMs (default is 10)',
+   limitrequestline => 'global LimitRequestLine setting (default is 128)',
+   perlpod          => 'location of perl pod documents (for testing downloads)',
+   proxyssl_url     => 'url for testing ProxyPass / https (default is localhost)',
+   sslca            => 'location of SSL CA (default is $t_conf/ssl/ca)',
+   sslcaorg         => 'SSL CA organization to use for tests (default is asf)',
+   sslproto         => 'SSL/TLS protocol version(s) to test',
+   libmodperl       => 'path to mod_perl\'s .so (full or relative to LIBEXECDIR)',
+   defines          => 'values to add as -D defines (for example, "VAR1 VAR2")',
    (map { $_ . '_module_name', "$_ module name"} qw(cgi ssl thread access auth php)),
 );
 
 my %filepath_conf_opts = map { $_ => 1 }
-    qw(top_dir t_dir t_conf t_logs t_pid_file t_conf_file src_dir serverroot
+    qw(top_dir t_dir t_conf t_logs t_state t_pid_file t_conf_file src_dir serverroot
        documentroot bindir sbindir httpd apxs httpd_conf httpd_conf_extra
        perlpod sslca libmodperl);
 
@@ -153,7 +157,7 @@ sub passenv {
 sub passenv_makestr {
     my @vars;
 
-    for (keys %passenv) {
+    for (sort keys %passenv) {
         push @vars, "$_=\$($_)";
     }
 
@@ -296,7 +300,16 @@ sub new {
     $vars->{t_conf}       ||= catfile $vars->{serverroot}, 'conf';
     $vars->{sslca}        ||= catfile $vars->{t_conf}, 'ssl', 'ca';
     $vars->{sslcaorg}     ||= 'asf';
+
+    if (!defined($vars->{sslproto}) and eval { require Apache::TestSSLCA; 1; }) {
+        $vars->{sslproto} = Apache::TestSSLCA::sslproto();
+    }
+    else {
+        $vars->{sslproto} ||= 'all';
+    }
+
     $vars->{t_logs}       ||= catfile $vars->{serverroot}, 'logs';
+    $vars->{t_state}      ||= catfile $vars->{serverroot}, 'state';
     $vars->{t_conf_file}  ||= catfile $vars->{t_conf},   'httpd.conf';
     $vars->{t_pid_file}   ||= catfile $vars->{t_logs},   'httpd.pid';
 
@@ -315,6 +328,7 @@ sub new {
     $vars->{group}        ||= $self->default_group;
     $vars->{serveradmin}  ||= $self->default_serveradmin;
 
+    $vars->{threadsperchild} ||= 10;
     $vars->{minclients}   ||= 1;
     $vars->{maxclients_preset} = $vars->{maxclients} || 0;
     # if maxclients wasn't explicitly passed try to
@@ -327,13 +341,36 @@ sub new {
         $vars->{maxclients_preset} < $vars->{minclients}) {
         $vars->{minclients} = $vars->{maxclients_preset};
     }
+    if ($vars->{minclients} < 2) {
+        $vars->{maxspare} = 2;
+    } else {
+        $vars->{maxspare} = $vars->{minclients};
+    }
+    if ($vars->{maxclients} < $vars->{maxspare} + 1) {
+        $vars->{maxclients} = $vars->{maxspare} + 1;
+    }
 
-    # for threaded mpms MaxClients must be a multiple of
-    # ThreadsPerChild (i.e. maxclients % minclients == 0)
-    # so unless -maxclients was explicitly specified use a double of
-    # minclients
-    $vars->{maxclientsthreadedmpm} =
-        $vars->{maxclients_preset} || $vars->{minclients} * 2;
+    # for threaded mpms MinClients and MaxClients must be a
+    # multiple of ThreadsPerChild
+    {
+        use integer;
+        $vars->{minclientsthreadedmpm} = ($vars->{minclients} + $vars->{threadsperchild} - 1) /
+            $vars->{threadsperchild} * $vars->{threadsperchild};
+        $vars->{maxclientsthreadedmpm} = ($vars->{maxclients} + $vars->{threadsperchild} - 1) /
+            $vars->{threadsperchild} * $vars->{threadsperchild};
+        $vars->{maxsparethreadedmpm} = ($vars->{maxspare} + $vars->{threadsperchild} - 1) /
+            $vars->{threadsperchild} * $vars->{threadsperchild};
+        $vars->{startserversthreadedmpm} = $vars->{minclientsthreadedmpm} / $vars->{threadsperchild};
+    }
+    if ($vars->{maxsparethreadedmpm} < 2 * $vars->{threadsperchild}) {
+        $vars->{maxsparethreadedmpm} = 2 * $vars->{threadsperchild};
+    }
+    if ($vars->{maxclientsthreadedmpm} < $vars->{maxsparethreadedmpm} + $vars->{threadsperchild}) {
+        $vars->{maxclientsthreadedmpm} = $vars->{maxsparethreadedmpm} + $vars->{threadsperchild};
+    }
+
+    $vars->{limitrequestline} ||= 128;
+    $vars->{limitrequestlinex2} = 2 * $vars->{limitrequestline};
 
     $vars->{proxy}        ||= 'off';
     $vars->{proxyssl_url} ||= '';
@@ -376,7 +413,7 @@ sub httpd_config {
             # not sure what else could go wrong but we can't continue
             die "something is wrong, mod_perl 2.0 build should have " .
                 "supplied all the needed information to run the tests. " .
-                "Please post lib/Apache/BuildConfig.pm along with the " .
+                "Please post lib/Apache2/BuildConfig.pm along with the " .
                 "bug report";
         }
 
@@ -513,6 +550,15 @@ sub configure_proxy {
         unless ($vars->{maxclients_preset}) {
             $vars->{minclients}++;
             $vars->{maxclients}++;
+            $vars->{maxspare}++;
+            $vars->{startserversthreadedmpm} ++;
+            $vars->{minclientsthreadedmpm} += $vars->{threadsperchild};
+            $vars->{maxclientsthreadedmpm} += $vars->{threadsperchild};
+            $vars->{maxsparethreadedmpm} += $vars->{threadsperchild};
+            #In addition allow for some backend processes
+            #in keep-alive state. For threaded MPMs we
+            #already should be fine.
+            $vars->{maxclients} += 3;
         }
         $vars->{proxy} = $self->{vhosts}->{'mod_proxy'}->{hostport};
         return $vars->{proxy};
@@ -1027,6 +1073,8 @@ sub perlscript_header {
         push @dirs, $dir if -d $dir;
     }
 
+    push @dirs, canonpath $FindBin::Bin;
+    
     my $dirs = join("\n    ", '', @dirs) . "\n";;
 
     return <<"EOF";
@@ -1124,7 +1172,7 @@ sub clean {
     $self->cmodules_clean;
     $self->sslca_clean;
 
-    for (keys %{ $self->{clean}->{files} }) {
+    for (sort keys %{ $self->{clean}->{files} }) {
         if (-e $_) {
             debug "unlink $_";
             unlink $_;
@@ -1396,6 +1444,15 @@ sub check_vars {
             unless ($vars->{maxclients_preset}) {
                 $vars->{minclients}++;
                 $vars->{maxclients}++;
+                $vars->{maxspare}++;
+                $vars->{startserversthreadedmpm} ++;
+                $vars->{minclientsthreadedmpm} += $vars->{threadsperchild};
+                $vars->{maxclientsthreadedmpm} += $vars->{threadsperchild};
+                $vars->{maxsparethreadedmpm} += $vars->{threadsperchild};
+                #In addition allow for some backend processes
+                #in keep-alive state. For threaded MPMs we
+                #already should be fine.
+                $vars->{maxclients} += 3;
             }
         }
     }
@@ -1558,6 +1615,7 @@ sub generate_httpd_conf {
     $self->generate_index_html;
 
     $self->gendir($vars->{t_logs});
+    $self->gendir($vars->{t_state});
     $self->gendir($vars->{t_conf});
 
     my @very_last_postamble = ();
@@ -1620,7 +1678,7 @@ sub generate_httpd_conf {
     # but wasn't included in the system-wide httpd.conf
 
     print $out "<IfModule mod_alias.c>\n";
-    for (keys %aliases) {
+    for (sort keys %aliases) {
         next unless $vars->{$aliases{$_}};
         print $out "    Alias /getfiles-$_ $vars->{$aliases{$_}}\n";
     }
@@ -1672,7 +1730,7 @@ sub need_reconfiguration {
     # last run and thus avoid the reconfiguration?
     {
         my $passenv = passenv();
-        if (my @env_vars = grep { $ENV{$_} } keys %$passenv) {
+        if (my @env_vars = sort grep { $ENV{$_} } keys %$passenv) {
             push @reasons, "environment variables (@env_vars) are set";
         }
     }
@@ -1724,6 +1782,11 @@ sub which {
 
     return undef unless $program;
 
+    # No need to search PATH components
+    # if $program already contains a path
+    return $program if !OSX and !WINFU and
+        $program =~ /\// and -f $program and -x $program;
+
     my @dirs = File::Spec->path();
 
     require Config;
@@ -2148,6 +2211,18 @@ PidFile     @t_pid_file@
 ErrorLog    @t_logs@/error_log
 LogLevel    debug
 
+<IfModule mod_version.c>
+<IfVersion > 2.4.1>
+    DefaultRunTimeDir "@t_logs@"
+    LogLevel trace8
+</IfVersion>
+<IfVersion > 2.4.34>
+<IfDirective DefaultStateDir>
+    DefaultStateDir "@t_state@"
+</IfDirective>
+</IfVersion>
+</IfModule>
+
 <IfModule mod_log_config.c>
     TransferLog @t_logs@/access_log
 </IfModule>
@@ -2174,10 +2249,10 @@ HostnameLookups Off
     LockFile             @t_logs@/accept.lock
 </IfVersion>
 </IfModule>
-    StartServers         1
-    MinSpareThreads      @MinClients@
-    MaxSpareThreads      @MinClients@
-    ThreadsPerChild      @MinClients@
+    StartServers         @StartServersThreadedMPM@
+    MinSpareThreads      @ThreadsPerChild@
+    MaxSpareThreads      @MaxSpareThreadedMPM@
+    ThreadsPerChild      @ThreadsPerChild@
     MaxClients           @MaxClientsThreadedMPM@
     MaxRequestsPerChild  0
 </IfModule>
@@ -2190,8 +2265,8 @@ HostnameLookups Off
 </IfModule>
     NumServers           1
     StartThreads         @MinClients@
-    MinSpareThreads      @MinClients@
-    MaxSpareThreads      @MinClients@
+    MinSpareThreads      1
+    MaxSpareThreads      @MaxSpare@
     MaxThreadsPerChild   @MaxClients@
     MaxRequestsPerChild  0
 </IfModule>
@@ -2203,8 +2278,8 @@ HostnameLookups Off
 </IfVersion>
 </IfModule>
     StartServers         @MinClients@
-    MinSpareServers      @MinClients@
-    MaxSpareServers      @MinClients@
+    MinSpareServers      1
+    MaxSpareServers      @MaxSpare@
     MaxClients           @MaxClients@
     MaxRequestsPerChild  0
 </IfModule>
@@ -2212,8 +2287,8 @@ HostnameLookups Off
 <IfDefine APACHE1>
     LockFile             @t_logs@/accept.lock
     StartServers         @MinClients@
-    MinSpareServers      @MinClients@
-    MaxSpareServers      @MinClients@
+    MinSpareServers      1
+    MaxSpareServers      @MaxSpare@
     MaxClients           @MaxClients@
     MaxRequestsPerChild  0
 </IfDefine>
