@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
 # this work for additional information regarding copyright ownership.
@@ -75,6 +76,7 @@ use constant OPENBSD => $^O eq 'openbsd';
 use constant WIN32   => $^O eq 'MSWin32';
 
 use constant MSVC => WIN32() && ($Config{cc} eq 'cl');
+use constant DMAKE => WIN32() && ($Config{make} eq 'dmake');
 
 use constant REQUIRE_ITHREADS => grep { $^O eq $_ } qw(MSWin32);
 use constant PERL_HAS_ITHREADS =>
@@ -141,13 +143,15 @@ sub httpd_is_source_tree {
         defined $prefix && -d $prefix && -e "$prefix/CHANGES";
 }
 
-# try to find the apxs utility, set $apxs to the path if found,
+# try to find the apxs utility, set $self->{MP_APXS} to the path if found,
 # otherwise to ''
-my $apxs; # undef so we know we haven't tried to set it yet
 sub find_apxs_util {
     my $self = shift;
 
-    $apxs = ''; # not found
+    if (not defined $self->{MP_APXS}) {
+        $self->{MP_APXS} = ''; # not found
+    }
+
     my @trys = ($Apache2::Build::APXS,
                 $self->{MP_APXS},
                 $ENV{MP_APXS});
@@ -177,7 +181,7 @@ sub find_apxs_util {
         next unless ($apxs_try = $_);
         chomp $apxs_try;
         if (-x $apxs_try) {
-            $apxs = $apxs_try;
+            $self->{MP_APXS} = $apxs_try;
             last;
         }
     }
@@ -207,7 +211,7 @@ sub ap_destdir {
 sub apxs {
     my $self = shift;
 
-    $self->find_apxs_util() unless defined $apxs;
+    $self->find_apxs_util() unless defined $self->{MP_APXS};
 
     my $is_query = (@_ == 2) && ($_[0] eq '-q');
 
@@ -221,7 +225,7 @@ sub apxs {
         }
     }
 
-    unless ($apxs) {
+    unless ($self->{MP_APXS}) {
         my $prefix = $self->{MP_AP_PREFIX} || "";
         return '' unless -d $prefix and $is_query;
         my $val = $apxs_query{$_[1]};
@@ -229,15 +233,15 @@ sub apxs {
     }
 
     my $devnull = devnull();
-    my $val = qx($apxs @_ 2>$devnull);
+    my $val = qx($self->{MP_APXS} @_ 2>$devnull);
     chomp $val if defined $val;
 
     unless ($val) {
         # do we have an error or is it just an empty value?
-        my $error = qx($apxs @_ 2>&1);
+        my $error = qx($self->{MP_APXS} @_ 2>&1);
         chomp $error if defined $error;
         if ($error) {
-            error "'$apxs @_' failed:";
+            error "'$self->{MP_APXS} @_' failed:";
             error $error;
         }
         else {
@@ -281,7 +285,7 @@ my %threaded_mpms = map { $_ => 1 }
 sub mpm_is_threaded {
     my $self = shift;
     my $mpm_name = $self->mpm_name();
-    return $threaded_mpms{$mpm_name} || 0;
+    return exists $threaded_mpms{$mpm_name} ? 1 : 0;
 }
 
 sub mpm_name {
@@ -289,6 +293,11 @@ sub mpm_name {
 
     return $self->{mpm_name} if $self->{mpm_name};
 
+    if ($self->httpd_version =~ /^(\d+)\.(\d+)\.(\d+)/) {
+	delete $threaded_mpms{dynamic} if $self->mp_nonthreaded_ok;
+	return $self->{mpm_name} = 'dynamic' if ($1*1000+$2)*1000+$3>=2003000;
+    }
+
     # XXX: hopefully apxs will work on win32 one day
     return $self->{mpm_name} = 'winnt' if WIN32;
 
@@ -602,6 +611,14 @@ sub ap_ccopts {
         $ccopts .= " -DMP_TRACE";
     }
 
+    if ($self->has_gcc_version('5.0.0') && $ccopts !~ /-fgnu89-inline/) {
+        $ccopts .= " -fgnu89-inline";
+    }
+
+    if ($self->has_clang && $ccopts !~ /-std=gnu89/) {
+        $ccopts .= " -std=gnu89";
+    }
+
     # make sure apr.h can be safely included
     # for example Perl's included -D_GNU_SOURCE implies
     # -D_LARGEFILE64_SOURCE on linux, but this won't happen on
@@ -609,6 +626,9 @@ sub ap_ccopts {
     my $extra_cppflags = $self->apxs_extra_cppflags;
     $ccopts .= " " . $extra_cppflags;
 
+    # Make sure the evil AP_DEBUG is not defined when building mod_perl
+    $ccopts =~ s/ ?-DAP_DEBUG\b//;
+
     $ccopts;
 }
 
@@ -629,6 +649,16 @@ sub has_gcc_version {
     return cmp_tuples(\@tuples, \@r_tuples) == 1;
 }
 
+sub has_clang {
+    my $self = shift;
+
+    my $has_version = $self->perl_config('gccversion');
+
+    return 0 unless $has_version;
+
+    return $has_version =~ m/Clang/;
+}
+
 sub cmp_tuples {
     my ($num_a, $num_b) = @_;
 
@@ -911,6 +941,7 @@ sub new {
     my $self = bless {
         cwd        => Cwd::fastcwd(),
         MP_LIBNAME => 'mod_perl',
+        MP_APXS    => undef, # so we know we haven't tried to set it yet
         @_,
     }, $class;
 
@@ -931,7 +962,7 @@ my %default_files = (
 
 sub clean_files {
     my $self = shift;
-    [map { $self->default_file($_) } keys %default_files];
+    [sort map { $self->default_file($_) } keys %default_files];
 }
 
 sub default_file {
@@ -1153,18 +1184,7 @@ sub apr_bindir {
 
 sub apr_generation {
     my ($self) = @_;
-
-    my $httpd_v = $self->httpd_version_as_int;
-
-    if ($httpd_v =~ m/2[4-9]\d+/) {
-        return 2;
-    }
-    elsif ($httpd_v =~ m/2[1-3]\d+/) {
-        return 1;
-    }
-    else {
-        return;
-    }
+    return $self->httpd_version_as_int =~ m/2[1-9]\d+/ ? 1 : 0;
 }
 
 # returns an array of apr/apu linking flags (--link-ld --libs) if found
@@ -1224,8 +1244,7 @@ sub apru_config_path {
         $self->{$key} = $self->{$mp_key};
     }
 
-    my $apr_generation = $self->apr_generation;
-    my $config = $apr_generation ? "$what-${apr_generation}-config" : "$what-config";
+    my $config = $self->apr_generation ? "$what-1-config" : "$what-config";
 
     if (!$self->{$key}) {
         my @tries = ();
@@ -1474,11 +1493,21 @@ sub get_apr_config {
 
     return $self->{apr_config} if $self->{apr_config};
 
+    my $fh;
     my $header = catfile $self->apr_includedir, "apr.h";
-    open my $fh, $header or do {
-        error "Unable to open $header: $!";
-        return undef;
-    };
+    if (WIN32) {
+        open $fh, $header or do {
+            error "Unable to open $header: $!";
+            return undef;
+        };
+    }
+    else {
+        my @command = ($self->perl_config('cpp'), '-dM', $header);
+        open $fh, '-|', @command or do {
+            error "Unable to preprocess $header with @command: $!";
+            return undef;
+        };
+    }
 
     my %cfg;
     while (<$fh>) {
@@ -1529,7 +1558,8 @@ sub make_xs {
     my @files;
     my @xs_targ;
 
-    while (my ($name, $xs) = each %{ $self->{XS} }) {
+    foreach my $name (sort keys %{ $self->{XS} }) {
+        my $xs = $self->{XS}->{$name};
         #Foo/Bar.xs => Foo_Bar.c
         (my $c = $xs) =~ s:.*?WrapXS/::;
         $c =~ s:/:_:g;
@@ -1573,7 +1603,7 @@ my $mm_replace = join '|', keys %perl_config_pm_alias;
 my %perl_config_pm_alias_values = reverse %perl_config_pm_alias;
 my @perl_config_pm_alias_values = keys %perl_config_pm_alias_values;
 
-my @perl_config_pm = (@perl_config_pm_alias_values, qw(cc cpprun
+my @perl_config_pm = sort(@perl_config_pm_alias_values, qw(cc cpprun
     rm ranlib lib_ext obj_ext cccdlflags lddlflags optimize));
 
 sub mm_replace {
@@ -1888,7 +1918,8 @@ EOI
     #XXX short-term compat for Apache::TestConfigPerl
     $libs{shared} = $libs{dso};
 
-    while (my ($type, $lib) = each %libs) {
+    foreach my $type (sort keys %libs) {
+        my $lib = $libs{$type};
         print $fh $self->canon_make_attr("lib_$type", $libs{$type});
     }
 
@@ -1942,7 +1973,7 @@ EOI
     }
 
     my @libs;
-    for my $type (map { uc } keys %libs) {
+    for my $type (sort map { uc } keys %libs) {
         next unless $self->{"MP_USE_$type"};
         # on win32 mod_perl.lib must come after mod_perl.so
         $type eq 'STATIC'
@@ -1979,6 +2010,11 @@ EOF
 
     print $fh $install;
 
+    print $fh <<'EOF' if DMAKE;
+
+.USESHELL :
+EOF
+
     print $fh <<'EOF';
 
 .SUFFIXES: .xs .c $(MODPERL_OBJ_EXT) .lo .i .s
@@ -2207,7 +2243,8 @@ sub has_large_files_conflict {
     # with it is that we didn't have such a case yet, but may need to
     # deal with it later
 
-    return $perl_lfs64 ^ $apr_lfs64;
+    return 0;
+    # $perl_lfs64 ^ $apr_lfs64;
 }
 
 # if perl is built with uselargefiles, but apr not, the build won't
