@@ -69,17 +69,15 @@ char *modperl_cmd_push_filter_handlers(MpAV **handlers,
      */
     if (*name == '-') {
         MP_TRACE_h(MP_FUNC,
-                   "[%s] warning: filter handler %s will be not autoloaded. "
+                   "warning: filter handler %s will be not autoloaded. "
                    "Unless the module defining this handler is explicitly "
-                   "preloaded, filter attributes will be ignored.\n",
-                   modperl_pid_tid(p), h->name);
+                   "preloaded, filter attributes will be ignored.");
     }
     else {
         MpHandlerAUTOLOAD_On(h);
         MP_TRACE_h(MP_FUNC,
-                   "[%s] filter handler %s will be autoloaded (to make "
-                   "the filter attributes available)\n",
-                   modperl_pid_tid(p), h->name);
+                   "filter handler %s will be autoloaded (to make "
+                   "the filter attributes available)", h->name);
     }
 
     if (!*handlers) {
@@ -417,6 +415,35 @@ MP_CMD_SRV_DECLARE(init_handlers)
     return modperl_cmd_post_read_request_handlers(parms, mconfig, arg);
 }
 
+#if AP_SERVER_MAJORVERSION_NUMBER>2 || \
+    (AP_SERVER_MAJORVERSION_NUMBER == 2 && AP_SERVER_MINORVERSION_NUMBER>=3)
+
+MP_CMD_SRV_DECLARE2(authz_provider)
+{
+    apr_pool_t *p = parms->pool;
+    char *name = apr_pstrdup(p, arg1);
+    char *cb = apr_pstrdup(p, arg2);
+
+    modperl_register_auth_provider_name(p, AUTHZ_PROVIDER_GROUP, name,
+                                        AUTHZ_PROVIDER_VERSION, cb, NULL,
+                                        AP_AUTH_INTERNAL_PER_CONF);
+    return NULL;
+}
+
+MP_CMD_SRV_DECLARE2(authn_provider)
+{
+    apr_pool_t *p = parms->pool;
+    char *name = apr_pstrdup(p, arg1);
+    char *cb = apr_pstrdup(p, arg2);
+
+    modperl_register_auth_provider_name(p, AUTHN_PROVIDER_GROUP, name,
+                                        AUTHN_PROVIDER_VERSION, cb, NULL,
+                                        AP_AUTH_INTERNAL_PER_CONF);
+    return NULL;
+}
+
+#endif
+
 static const char *modperl_cmd_parse_args(apr_pool_t *p,
                                           const char *args,
                                           apr_table_t **t)
@@ -558,6 +585,11 @@ MP_CMD_SRV_DECLARE(perldo)
                           arg, NULL);
     }
 
+#ifdef USE_ITHREADS
+    MP_TRACE_i(MP_FUNC, "using interp %lx to execute perl section:\n%s",
+               scfg->mip->parent, arg);
+#endif
+
     {
         SV *server = MP_PERLSECTIONS_SERVER_SV;
         SV *code = newSVpv(arg, 0);
@@ -763,58 +795,6 @@ MP_CMD_SRV_DECLARE_FLAG(setup_env)
 
 #ifdef USE_ITHREADS
 
-#define MP_INTERP_SCOPE_USAGE "PerlInterpScope must be one of "
-
-#define MP_INTERP_SCOPE_DIR_OPTS \
-    "handler, subrequest or request"
-
-#define MP_INTERP_SCOPE_DIR_USAGE \
-    MP_INTERP_SCOPE_USAGE MP_INTERP_SCOPE_DIR_OPTS
-
-#define MP_INTERP_SCOPE_SRV_OPTS \
-    "connection, " MP_INTERP_SCOPE_DIR_OPTS
-
-#define MP_INTERP_SCOPE_SRV_USAGE \
-    MP_INTERP_SCOPE_USAGE MP_INTERP_SCOPE_SRV_OPTS
-
-MP_CMD_SRV_DECLARE(interp_scope)
-{
-    modperl_interp_scope_e *scope;
-    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
-    MP_dSCFG(parms->server);
-    int is_per_dir = parms->path ? 1 : 0;
-
-    scope = is_per_dir ? &dcfg->interp_scope : &scfg->interp_scope;
-
-    switch (toLOWER(*arg)) {
-      case 'h':
-        if (strcaseEQ(arg, "handler")) {
-            *scope = MP_INTERP_SCOPE_HANDLER;
-            break;
-        }
-      case 's':
-        if (strcaseEQ(arg, "subrequest")) {
-            *scope = MP_INTERP_SCOPE_SUBREQUEST;
-            break;
-        }
-      case 'r':
-        if (strcaseEQ(arg, "request")) {
-            *scope = MP_INTERP_SCOPE_REQUEST;
-            break;
-        }
-      case 'c':
-        if (!is_per_dir && strcaseEQ(arg, "connection")) {
-            *scope = MP_INTERP_SCOPE_CONNECTION;
-            break;
-        }
-      default:
-        return is_per_dir ?
-             MP_INTERP_SCOPE_DIR_USAGE : MP_INTERP_SCOPE_SRV_USAGE;
-    };
-
-    return NULL;
-}
-
 #define MP_CMD_INTERP_POOL_IMP(xitem) \
 const char *modperl_cmd_interp_##xitem(cmd_parms *parms, \
                                       void *mconfig, const char *arg) \
@@ -833,3 +813,10 @@ MP_CMD_INTERP_POOL_IMP(min_spare);
 MP_CMD_INTERP_POOL_IMP(max_requests);
 
 #endif /* USE_ITHREADS */
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
