@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestFilter::in_str_bin_data;
 
 # test that $r->print and $f->print handle binary data correctly
@@ -6,13 +7,15 @@ package TestFilter::in_str_bin_data;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::RequestIO ();
-use Apache::RequestRec ();
-use Apache::Filter ();
+use Apache2::RequestIO ();
+use Apache2::RequestRec ();
+use Apache2::Filter ();
 
 use Apache::TestTrace;
 
-use Apache::Const -compile => qw(OK M_POST);
+use TestCommon::Utils ();
+
+use Apache2::Const -compile => qw(OK M_POST);
 
 sub pass_through {
     my $f = shift;
@@ -22,20 +25,20 @@ sub pass_through {
         $f->print($buffer);
     }
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 sub handler {
     my $r = shift;
 
-    if ($r->method_number == Apache::M_POST) {
-        my $data = ModPerl::Test::read_post($r);
+    if ($r->method_number == Apache2::Const::M_POST) {
+        my $data = TestCommon::Utils::read_post($r);
         my $length = length $data;
         debug "pass through $length bytes of $data\n";
         $r->print($data);
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
