@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestModperl::setupenv;
 
 use strict;
@@ -181,9 +182,9 @@ __DATA__
 
     KeepAlive On
 
-    <IfDefine PERL_USEITHREADS>
-        PerlInterpScope connection
-    </Ifdefine>
+#    <IfDefine PERL_USEITHREADS>
+#        PerlInterpScope connection
+#    </Ifdefine>
 
     PerlModule TestModperl::setupenv
 
